package com.company.generatedcolumns.web.screens.year;

import com.company.generatedcolumns.entity.Month;
import com.company.generatedcolumns.entity.Year;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.gui.UiComponents;
import com.haulmont.cuba.gui.components.Action;
import com.haulmont.cuba.gui.components.Component;
import com.haulmont.cuba.gui.components.HasValue;
import com.haulmont.cuba.gui.components.LookupField;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.components.data.ValueSource;
import com.haulmont.cuba.gui.components.data.options.ContainerOptions;
import com.haulmont.cuba.gui.components.data.value.ContainerValueSource;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.InstanceContainer;
import com.haulmont.cuba.gui.screen.EditedEntityContainer;
import com.haulmont.cuba.gui.screen.LoadDataBeforeShow;
import com.haulmont.cuba.gui.screen.StandardEditor;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@UiController("generatedcolumns_Year.edit")
@UiDescriptor("year-edit.xml")
@EditedEntityContainer("yearDc")
@LoadDataBeforeShow
public class YearEdit extends StandardEditor<Year> {

    private static final String MONTH_31 = "ends with 31";
    private static final String MONTH_NOT_31 = "the-rest";

    private static final List<Month> MONTHS = new ArrayList<>();

    @Inject private UiComponents uiComponents;
    @Inject private DataManager dataManager;

    @Inject private Table<Month> monthsTable;
    @Inject private CollectionContainer<Month> monthsDc;
    @Inject private CollectionContainer<Month> selectionMonthsDl;


    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        buildTransientMonthRecords();
    }

    private void buildTransientMonthRecords() {
        int order = 0;
        MONTHS.add(create(order++, "January", 31));
        MONTHS.add(create(order++, "February", 28));
        MONTHS.add(create(order++, "March", 31));
        MONTHS.add(create(order++, "April", 30));
        MONTHS.add(create(order++, "May", 31));
        MONTHS.add(create(order++, "Jun", 30));
        MONTHS.add(create(order++, "July", 31));
        MONTHS.add(create(order++, "August", 31));
        MONTHS.add(create(order++, "September", 30));
        MONTHS.add(create(order++, "Octuber", 31));
        MONTHS.add(create(order++, "November", 30));
        MONTHS.add(create(order++, "December", 31));
    }

    private Month create(int order, String name, int lastDay) {
        Month month = dataManager.create(Month.class);
        month.setOrderNumber(order);
        month.setName(name);
        month.setLastDay(lastDay);
        return month;
    }


    public Component generateMonthTypeColumn(Month entity) {
        InstanceContainer<Month> container = monthsTable.getInstanceContainer(entity);
        LookupField<String> lookupField = uiComponents.create(LookupField.NAME);
        lookupField.setValueSource(new ContainerValueSource<>(container, "name"));
        lookupField.setOptionsList(Arrays.asList(MONTH_31, MONTH_NOT_31));
        lookupField.setWidthFull();
        lookupField.addValueChangeListener(event-> updateMonthSelection(event, entity));
        return lookupField;
    }

    private void updateMonthSelection(HasValue.ValueChangeEvent<String> valueChangeEvent, Month currentInstance) {
        selectionMonthsDl.getMutableItems().clear();
        if (MONTH_31.equals(valueChangeEvent.getValue())) {
            selectionMonthsDl.getMutableItems().addAll(MONTHS.stream()
                                                             .filter(month -> month.getLastDay() == 31)
                                                             .collect(Collectors.toList()));
        } else if (MONTH_NOT_31.equals(valueChangeEvent.getValue())) {
            selectionMonthsDl.getMutableItems().addAll(MONTHS.stream()
                                                             .filter(month -> month.getLastDay() < 31)
                                                             .collect(Collectors.toList()));
        }
        currentInstance.setSelectedMonth(null);
    }

    public Component generatedMonthColumn(Month entity) {
        InstanceContainer<Month> container = monthsTable.getInstanceContainer(entity);
        LookupField<Month> lookupField = uiComponents.create(LookupField.NAME);
        lookupField.setValueSource(new ContainerValueSource<>(container, "selectedMonth"));
        lookupField.setOptions(new ContainerOptions<>(selectionMonthsDl));
        lookupField.setWidthFull();
        return lookupField;
    }

    @Subscribe("monthsTable.add")
    public void onMonthsTableAdd(Action.ActionPerformedEvent event) {
        Month month = dataManager.create(Month.class);
        month.setYear(getEditedEntity());
        monthsDc.getMutableItems().add(month);
    }

}